/*
  JAH controller v.1
  JustAnotherHydroFarm
  addidis@gmail.com
  https://hackaday.io/project/167510-jah

  The idea is to use a long bitfield like old 24 hour mechanical timers.
  Each bit represents one of the tact switches you would flip on the
  mechanical timers. This is calculated by providing the hour the lights
  are intended to go on and how long you want the lights to stay on in
  each phase.

  Ideally, you will start a cycle and the controller will run to conclusion
  with out losing power (8-12weeks or more?). If the power is lost using the approach above
  simplifies recovering midway after powerloss.

  The circuit:
  mega2560

  \
  SD card attached to SPI bus as follows:
** MOSI - pin 51
** MISO - pin 50
** CLK - pin 52
** CS - pin 53

  RTC ds1307 as follows
   sda  - 20
   scl  - 21

** buttons
   A8 output
   A9 INPUT 1
   A10 INPUT 2
   A11 INPUT 3
   A12 INPUT 4
   A13 INPUT 5
   A14 INPUT 6
   A15 OUTPUT +

  rj45 jack pins OUT OF ORDER, SEE DECLARES
   8 GROUND OUTPUT PIN #23
   7 CH1 25
   6 CH2 27
   5 CH4 29
   4 CH5 31
   3 CH3 33
   2 NO CONNECTION
   1 NO CONNECTION

** NRF24l01+

*/

#include <SPI.h>
#include "SdFat.h"
#include <RTClib.h>
#include <RF24.h>
#include <WiFi.h>
#include <Adafruit_GFX.h>
#include <MCUFRIEND_kbv.h>
#include <WiFiUdp.h>
#include <time.h>


//////////////////
//Hardware Defines
//////////////////
//#define CST -6
//#define DST -5
#define TFT24             //initialized but not really implimented yet oled is primary display currently
//#define serial          //define for serial output, comment out to remove serial output
#define oled              //define for oled output, comment out to remove OLED output
//#define datalog         //uncomment to /turn on datalogging/  be dissapointed because it too is not in yet.
#define MEGA_SOFT_SPI 1   //i think this is done in sd.h files , check later if it can be removed. 

//butonses
#define esc A14           //escape menu  49 = select
#define up A11            //menu up  47 = down
#define down A12          //menu down 45 = left
#define left A13          //menu left 43 = right
#define right A10         //menu right  41 = up
#define select A9         //menu select 39 esc 

//ssd ARRAY
#define SSDch1 25
#define SSDch2 27
#define SSDch3 33
#define SSDch4 29
#define SSDch5 31

//TFT shield with sd
#ifdef TFT24
#define BLACK   0x0000
#define BLUE    0x001F
#define RED     0xF800
#define GREEN   0x07E0
#define CYAN    0x07FF
#define MAGENTA 0xF81F
#define YELLOW  0xFFE0
#define WHITE   0xFFFF

// work in line numbers.  Font height in ht
int16_t ht = 16, top = 3, line, lines = 15, scroll;
MCUFRIEND_kbv tft;
#endif

//oled defines
#ifdef oled
#include <Adafruit_GFX.h>
#include <Adafruit_SSD1306.h>
#define BACKCOLOR 0x0000    // Black
#define PIXELCOLOR 0xFFFF   // White
#define SCREEN_WIDTH 128    // OLED display width, in pixels
#define SCREEN_HEIGHT 64    // OLED display height, in pixels
// Declaration for an SSD1306 display connected to I2C (SDA, SCL pins)
#define OLED_RESET     16   // Reset pin # (or -1 if sharing Arduino reset pin)
//Oled used
Adafruit_SSD1306 display(SCREEN_WIDTH, SCREEN_HEIGHT, &Wire, OLED_RESET);
#define LOGO_HEIGHT   52
#define LOGO_WIDTH    128
const unsigned char PROGMEM logo_bmp[] = {};
#endif

//RTC used
RTC_DS1307 rtc;

//nrf24l01
#define CE 16
#define CSN 53
RF24 radio(CE, CSN); // CE, CSN
const byte nodeAddress[5] = {'N', 'O', 'D', 'E', '1'};                            // setup radio pipe address for remote sensor node
int remoteNodeData[2] = {1, 1,};                                                  // integer array for slave node data:[node_id, returned_count]
//esp01
#define esptrigger  17  //gp0 


///////////////////
//////////////////
//Variable define
////////////////
///////////////

//rtc defines
char daysOfTheWeek[7][12] = {"Sunday", "Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday"};
int datalog = 0;
int oledposition = 0;
int days = 0;
int mins = 0;
long bitfield = 0 ;
const byte address[6] = "00001";

//sd
#if ENABLE_SOFTWARE_SPI_CLASS             // Must be set in SdFat/SdFatConfig.h

const uint8_t SOFT_MISO_PIN = 12;         // Pin numbers in templates must be constants.
const uint8_t SOFT_MOSI_PIN = 11;
const uint8_t SOFT_SCK_PIN  = 13;
const uint8_t SD_CHIP_SELECT_PIN = 10;
SdFatSoftSpi<SOFT_MISO_PIN, SOFT_MOSI_PIN, SOFT_SCK_PIN> sd;          // SdFat software SPI template
SdFile file;          // Test file.
#endif

//Phase duration variables
volatile byte SSDstatus = 0b00000000;           //BITflags:    (8)Flower, (7)veg,  (6)Germ,  (5)ch1/lights, (4)ch2/ventilation,  (3)ch3,  (2)ch4,  (1)ch5,

int Lightonhr = 8 ;                               //the hour the lights go on , best practice is dawn till dusk

int Lightflowerduration = 12;                     //standard defaults , which can be overwritten by SD input
int totalweeksflower = 8;

int Lightvegduration = 18;
int totalweeksveg = 6;

int Lightgermduration  = 16;
int totalweeksgerm = 2;

int lightonhr = 0;
int lightonduration  = 0;  // in hours

int timelefthr;
int timeleftmin;
int timeleftsec;

//display vars
int page = 0;
int selection = 1;
int maxmenu = 3;

struct tm timeinfo;
int timezone = -6;
volatile int militarytime = 0;

volatile static float temp1, humid1, press1, alt1, temp2, humid2, press2, temp3, humid3, press3, temp4, humid4, press4;

const byte numChars = 21;
char receivedChars[numChars];
char tempChars[numChars];        // temporary array for use when parsing
char messageFromPC[numChars] = {0};
volatile boolean newData = false;

volatile int toggletempF = 1;
const long interval = 10000;              // sw wdt for incoming sensor datum
unsigned long previousMillis = 0;        // will store last temp was read
/////////////
////////////
//Setup
///////////
//////////

void setup() {
  //Open serial communications and wait for port to open:
  Serial.begin(19200);         //serial 1 init
  Serial3.begin(19200);        //serial 2 init
  while (!Serial) {
    ; // wait for serial port to connect. Needed for native USB port only
  }

  //all teh pinses
  pinMode(13, OUTPUT);                        //onboard led
  digitalWrite(13, LOW);
  pinMode(esptrigger, OUTPUT);                //esp gpo NIST trigger
  digitalWrite(esptrigger, 1);
  pinMode(SD_CHIP_SELECT_PIN, OUTPUT);
  digitalWrite(SD_CHIP_SELECT_PIN, HIGH);

  pinMode(CE, OUTPUT);                        //set mode of ce and csn
  digitalWrite(CE, LOW);
  pinMode(CSN, OUTPUT);
  digitalWrite(CSN, LOW);

  pinMode(A8, OUTPUT);                        //power switches from io
  digitalWrite(A8, HIGH);                     // -
  pinMode(A15, OUTPUT);                       //power switches from io
  digitalWrite(A15, LOW);                     //+
  pinMode(select, INPUT);
  pinMode(right, INPUT);
  pinMode(left, INPUT);
  pinMode(down, INPUT);
  pinMode(up, INPUT);
  pinMode(esc, INPUT);

  pinMode(23, OUTPUT);                      //power SSDio from io
  digitalWrite(23, LOW);                    //-   only needs - + comes from other io
  pinMode(SSDch1, OUTPUT);
  digitalWrite(SSDch1, LOW);
  pinMode(SSDch2, OUTPUT);
  digitalWrite(SSDch2, LOW);
  pinMode(SSDch3, OUTPUT);
  digitalWrite(SSDch3, LOW);
  pinMode(SSDch4, OUTPUT);
  digitalWrite(SSDch4, LOW);
  pinMode(SSDch5, OUTPUT);
  digitalWrite(SSDch5, LOW);

  pinMode(10, OUTPUT);                //spi ss
  digitalWrite(10, HIGH);             // spi ss

  display.setTextSize(1);             // Normal 1:1 pixel scale
  display.setTextColor(WHITE);        // Draw white text
  display.setCursor(0, 0);            // Start at top-left corner

  // SSD1306_SWITCHCAPVCC = generate display voltage from 3.3V internally
  if (!display.begin(SSD1306_SWITCHCAPVCC, 0x3C)) {                               //address for display
    Serial.println(F("SSD1306 allocation failed"));
    for (;;);      // Don't proceed, loop forever
  }
  display.clearDisplay();
  display.display();

  //start tft
#ifdef TFT24
  tft.reset();
  uint16_t id = tft.readID();
  tft.begin(id);
  tft.setRotation(0);     //Portrait
  tft.fillScreen(RED);
  tft.setTextColor(WHITE, BLUE);
  tft.setTextSize(2);     // System font is 8 pixels.  ht = 8*2=16
  tft.setCursor(0, 0);
  tft.print("  J A H controller  ");
#endif

  //start RTC
  if (! rtc.begin()) {
    Serial.println("Couldn't find RTC");
    //while (1);
  }
  else {
    Serial.println("RTC found");
  }
  if (rtc.isrunning()) {
    display.setCursor(0, oledposition);
    display.println(F("RTC . . . OK"));
    display.display();
    //rtc.adjust(DateTime(F(__DATE__), F(__TIME__)));   //uncomment this if you set the rtc wrong and wanna reset it by programming it.
    oledposition = oledposition + 8;
  }
  if (! rtc.isrunning()) {
    Serial.println("RTC is NOT !running!");
    display.setCursor(0, oledposition);
    display.println(F("RTC . . . NOT OK"));
    display.display();
    oledposition = oledposition + 8;
    // following line sets the RTC to the date & time this sketch was compiled
    //this no longer sets the rtc, the ntp does.
    //rtc.adjust(DateTime(F(__DATE__), F(__TIME__)));     //this works if its not running.
  }

#ifdef TFT24
  //init SD
  Serial.print("Initializing SD card...");
  // see if the card is present and can be initialized:
  if (!sd.begin(SD_CHIP_SELECT_PIN)) {
    sd.initErrorHalt();
  }

  if (!file.open("SoftSPI.txt", O_RDWR | O_CREAT)) {
    display.println("Open failed");
    display.display();
    oledposition = oledposition + 8;
  }
  file.println(F("This line was printed using software SPI."));
  file.close();

  Serial.println("Card initialized.");
  display.setCursor(0, oledposition);
  display.print("SD  . . . OK");
  display.display();
  oledposition = oledposition + 8;
#endif

  //nrf24l01+ for sensors
  if (radio.begin()) {
    radio.openWritingPipe(address);
    radio.setPALevel(RF24_PA_MIN);
    radio.setDataRate(RF24_250KBPS);
    // set radio channel to use - ensure all slaves match this
    radio.setChannel(0x76);
    // set time between retries and max no. of retries
    radio.setRetries(4, 10);
    // enable ack payload - each slave replies with sensor data using this feature
    radio.enableAckPayload();
    radio.stopListening();
    display.setCursor(0, oledposition);
    display.println(F("NRF24l01. OK"));
    display.display();
    oledposition = oledposition + 8;
  } else {
    display.setCursor(0, oledposition);
    display.println(F("NRF24l01 already"));
    display.println(F("running?"));
    display.display();
    oledposition = oledposition + 8;
  }

  //check for schedule data on SD
  if (scheduleonSD()) {                                                        // if theres stuff on the sd lets use it instead of defaults.
    //not implimented yet building up around default values
  }
  else if (!scheduleonSD()) {                                                   //otherwise load the defaults
    if (bitRead(SSDstatus, 7)) {
      //set default  if its flower
      lightonhr = Lightonhr;
      lightonduration  = Lightflowerduration;   // in hours

    }
    if (bitRead(SSDstatus, 6)) {
      //set default  if its veg
      lightonhr = Lightonhr;
      lightonduration  = Lightvegduration;   // in hours
    }
    if (bitRead(SSDstatus, 5)) {
      //set default  if its germ
      lightonhr = Lightonhr;
      lightonduration  = Lightgermduration;   // in hours
    }
  }

  setbitfield();
  checkstatus();
  delay(2000);
}

void loop() {
  // echoserial3(); //debug
  checkstatus();
  drawScreen();
  buttonservice();

  recvWithStartEndMarkers();

  unsigned long currentMillis = millis();
 
  if(currentMillis - previousMillis >= interval) {
    // save the last time you read the sensor 
    previousMillis = currentMillis;   
temp1=0;
humid1=0;
  }
  if (newData == true) {
    strcpy(tempChars, receivedChars);
    // this temporary copy is necessary to protect the original data
    //   because strtok() used in parseData() replaces the commas with \0
    parseData();
    previousMillis = currentMillis;   //reset sw wdt on incoming datum
    newData = false;
  }
  //
}


void recvWithStartEndMarkers() {
  static boolean recvInProgress = false;
  static byte ndx = 0;
  char startMarker = '<';
  char endMarker = '>';
  char rc;

  while (Serial3.available() > 0 && newData == false) {
    rc = Serial3.read();

    if (recvInProgress == true) {
      if (rc != endMarker) {
        receivedChars[ndx] = rc;
        ndx++;
        if (ndx >= numChars) {
          ndx = numChars - 1;
        }
      }
      else {
        receivedChars[ndx] = '\0'; // terminate the string
        recvInProgress = false;
        ndx = 0;
        newData = true;

      }
    }

    else if (rc = startMarker) {
      recvInProgress = true;
      //
    }
  }
}


void parseData() {      // split the data into its parts

  char * strtokIndx; // this is used by strtok() as an index
  strtokIndx = strtok(tempChars, ",");     // get the first part - the string
  strcpy(messageFromPC, strtokIndx); // copy it to messageFromPC

  strtokIndx = strtok(NULL, ",");
  temp1 = atof(strtokIndx);     // convert this part to a float

  strtokIndx = strtok(NULL, ">");
  humid1 = atof(strtokIndx);     // convert this part to a float
  //Serial.println(temp1,1);
  //
}


void gettime(void) {
  delay(4000);                                          //rather than make it complicated just wait 4 seconds so user cant get banned from nist server for spamming.
  digitalWrite(esptrigger, LOW);
  display.clearDisplay();
  display.display();

  char  x = 0;
  char hours[2];
  int hoursint;
  char mins[2];
  int minsint;
  char secs[2];
  int secsint;
  char months[2];
  int monthsint;
  char days[2];
  int daysint;
  char years[4];
  int yearsint;

  // get incoming byte:
  while (x != '|') {
    x = Serial3.read();
    Serial.print(x);
  }
  digitalWrite(esptrigger, HIGH );

  x = Serial3.read();                                               //hour digit 1
  hours[0] = x;
  Serial.println(x);

  x = Serial3.read();                                               //hour digit 2
  if (x != ':') {
    hours[1] = x;
  } else {
    Serial.println(x);
    hours[1] = hours[0];
    hours[0] = 0;
  }
  Serial.println(x);
  hoursint = (hours[0] - '0') * 10 + (hours[1] - '0');
  Serial.println(hoursint, DEC);
  x = Serial3.read();                                              //min digit 1
  if (x = ':') {
    Serial.println(x);
    x = Serial3.read();
  }
  mins[0] = x;
  Serial.println(x);
  x = Serial3.read();                                             //min digit 2
  if (x != ':') {
    mins[1] = x;
  } else {
    Serial.println(x);
    mins[1] = mins[0];
    mins[0] = 0;
  }
  Serial.println(x);
  minsint = (mins[0] - '0') * 10 + (mins[1] - '0');
  Serial.println(minsint, DEC);

  x = Serial3.read();                                            //sec digit 1
  if (x = ':') {
    Serial.println(x);
    x = Serial3.read();
  }
  secs[0] = x;
  Serial.println(x);
  x = Serial3.read();                                           //sec digit 2
  if (x != ':') {
    secs[1] = x;
  } else {
    Serial.println(x);
    secs[1] = secs[0];
    secs[0] = 0;
  }
  Serial.println(x);
  secsint = (secs[0] - '0') * 10 + (secs[1] - '0');
  Serial.println(secsint, DEC);

  while (x != '|') {
    x = Serial3.read();
  }

  x = Serial3.read();                                           //days digit 1
  days[0] = x;
  x = Serial3.read();                                           //days digit 2
  if (x != '.') {
    days[1] = x;
  } else {
    days[1] =  days[0];
    days[0] = 0;
  }
  daysint = (days[0] - '0') * 10 + (days[1] - '0');
  Serial.println( daysint, DEC);

  x = Serial3.read();                                           //month digit 1
  if (x = '.') {
    Serial.println(x);
    x = Serial3.read();
  }
  months[0] = x;
  x = Serial3.read();                                           //month digit 2
  if (x != '.') {
    months[1] = x;
  } else {
    months[1] = months[0];
    months[0] = 0;
  }
  monthsint = (months[0] - '0') * 10 + (months[1] - '0');
  Serial.println(monthsint, DEC);

  x = Serial3.read();                                           //year digit 1
  if (x = '.') {
    x = Serial3.read();
  }
  years[0] = x;
  x = Serial3.read();                                           //year digit 2
  years[1] = x;
  x = Serial3.read();                                           //year digit 3
  years[2] = x;
  x = Serial3.read();                                           //year digit 4
  years[3] = x;
  yearsint = (years[0] - '0') * 1000 + (years[1] - '0') * 100 + (years[2] - '0') * 10 + (years[3] - '0');

  rtc.adjust(DateTime(yearsint, monthsint, daysint, (hoursint + timezone), minsint, secsint));
  page = 0;
}


void increaseTimezone(void) {
  timezone ++;
  if (timezone > 14) {
    timezone = -12;
  }
}


void decreaseTimezone(void) {
  timezone --;
  if (timezone < -12) {
    timezone = 14;
  }
}


void setbitfield(void) {
  bitfield = 0;
  int offtime = lightonhr + lightonduration;

  if (offtime <= 24) {                                                          // no roll over simple maths
    for (int i = 0; i <= 24; i++) {                                             //set bits light is on from start hour to finish hour, bitfield is initialized to 0
      if (i >= lightonhr && i <= lightonhr + lightonduration - 1) {
        bitSet(bitfield, i);
      }
    }
  } else {//process result for roll over
    if (offtime > 23) {
      offtime = offtime - 24;
    }
    if (lightonhr > offtime) {                                                   //When it rolls over we will set it a little differently.
      for (int i = 0; i < offtime; i++) {                                        //roll over, set 1 thru offtime
        bitSet(bitfield, i);
      }
      for (int i = 24; i >= lightonhr; i--) {                                     // then set 24 back to ontime
        bitSet(bitfield, i);
      }
    }
  }
  /*
    //print the resulting bitfield for debug.
      display.setCursor(0, 54);
      for (int i = 0; i <= 24; i++) {
      int a = bitRead(bitfield, i);
      Serial.print(a, DEC);
      display.print(a, DEC);
      }
      display.display();   //
  */
}


void  echoserial3(void) {
  if (Serial3.available()) {
    Serial.write(Serial3.read());
    delay(10);
  }
  if (Serial.available()) {
    Serial3.write(Serial.read());
    delay(10);
  }
}


int scheduleonSD(void) {
  int a = 0;   //sd data not implimented
  return a;

  //check for overrides stored on sd
}


void startCycle(void) {
  page = 3;
  lightonhr = Lightonhr;
}


void advanceCycle(void) {
  if (bitRead(SSDstatus, 7)) {
    bitClear(SSDstatus, 7);
    //end grow cycle , append datalog

  } else if (bitRead(SSDstatus, 6)) {
    bitClear(SSDstatus, 6);
    bitSet(SSDstatus, 7 );
    lightonduration  = Lightflowerduration;   // in hours
  } else if (bitRead(SSDstatus, 5)) {
    bitClear(SSDstatus, 5);
    bitSet(SSDstatus, 6);
    lightonduration  = Lightvegduration;   // in hours
  }
}


void rewindCycle(void) {
  if (bitRead(SSDstatus, 6)) {
    bitClear(SSDstatus, 6);
    bitSet(SSDstatus, 5);
    lightonduration  = Lightgermduration;   // in hours
  } else if (bitRead(SSDstatus, 7)) {
    bitClear(SSDstatus, 7);
    bitSet(SSDstatus, 6);
    lightonduration  = Lightvegduration;   // in hours
  }
}


void checkstatus(void) {                                                                          //recovering status on power loss / power up mid way  thru a cycle.
  DateTime now = rtc.now();   //pull time
  if (bitRead(bitfield, now.hour()) == 1) {                                                        //if it is light on in bitfield
    bitSet(SSDstatus, 4);                                                                         //set the statur bit
    digitalWrite(13, HIGH);                                                                       //turn on led to indicate lights on
    digitalWrite(SSDch5, HIGH);                                                                   //turn on SSD for lights
  } else {
    bitClear(SSDstatus, 4);
    digitalWrite(13, LOW);
    digitalWrite(SSDch5, LOW);
  }
}


void buttonservice(void)
{
  display.setCursor(0, 14);
  if (digitalRead(esc)) {
    display.print("ESC");
    display.display();
    if (page = 1) {
      page = 0;
    } else
      page = 1;
    delay(500);
  }

  if (digitalRead(up)) {
    display.print("Up");
    display.display();
    selection--;
    delay(500);
  }

  if (digitalRead(down)) {
    display.print("Down");
    display.display();
    selection++;
    delay(500);
  }

  if (digitalRead(left)) {
    display.print("Left");
    display.display();
    delay(500);
    lightonhr--;
    if (lightonhr < 0) {
      lightonhr = 23;
    }
    setbitfield();
  }

  if (digitalRead(right)) {
    display.print("Right");
    display.display();
    delay(500);
    lightonhr++;
    if (lightonhr == 24) {
      lightonhr = 0;
    }
    setbitfield();
  }

  if (digitalRead(select)) {
    display.print("Select");
    display.display();
    delay(500);
    switch (page)
    {
      case 0: page = 1;
        return;
      case 1:   //Main Menu
        switch (selection)
        {
          case 1: page = 2; //phase
            selection = 1;
            return;
          case 2: page = 4; //
            selection = 1;
            return;
          case 3: page = 5;
            selection = 1;
            return;
          case 4: page = 6;
            selection = 1;
            return;
        }
      case 2: //Phase
        switch (selection)
        {
          case 1:
            startCycle();
            //page = 3;
            return;
          case 2:
            advanceCycle();
            page = 0;
            return;
          case 3:
            rewindCycle();
            page = 0;
            return;
          case 4:
            harvest();
            page = 0;
            return;
        }
      case 3://startCycle
        switch (selection)
        {
          case 1:
            bitSet(SSDstatus, 6);
            page = 0;
            lightonduration  = Lightvegduration;   // in hours
            writestartfile();
            setbitfield();
            checkstatus();
            return;
          case 2:
            bitSet(SSDstatus, 5);
            page = 0;
            lightonduration  = Lightgermduration;   // in hours
            writestartfile();
            setbitfield();
            checkstatus();
            return;
        }
      case 4://drawclock
        switch (selection)
        {
          case 1:
            gettime();
            //settime();
            return;
          case 2:
            page = 7;
            selection = 1;
            return;
          case 3:
            militarytime  = 1 - militarytime;
            return;
          case 4:
            page = 0;
            selection = 1;
            return;
        }
      case 5://datalog
        {
          return;
        }
      case 6: //sensors
        switch (selection)
        {
          case 1:
            toggletempF = 1 - toggletempF;
            return;
        }
      case 7: //timezone
        switch (selection)
        {
          case 1:
            increaseTimezone();
            return;
          case 2:
            decreaseTimezone();
            return;
          case 3:
            page = 0;
            return;
        }
      default://main switch case
        //write default select action
        return;
    }
  }
}


void harvest(void) {
  bitClear(SSDstatus, 7);
  bitClear(SSDstatus, 6);
  bitClear(SSDstatus, 5);
}


void writestartfile(void) {
  DateTime now = rtc.now();   //pull time
  unsigned long start_time = now.secondstime();

  if (!sd.begin(SD_CHIP_SELECT_PIN)) {
    sd.initErrorHalt();
  }

  if (!file.open("setup.txt", O_RDWR | O_CREAT)) {
    sd.errorHalt(F("open failed"));
  }
  file.println(start_time, DEC);
  file.print(",");
  file.println(Lightgermduration, DEC);
  file.println(totalweeksgerm, DEC);
  file.println(Lightvegduration, DEC);
  file.println(totalweeksveg, DEC);
  file.println(Lightflowerduration, DEC);
  file.println(totalweeksflower, DEC);
  file.println(SSDstatus, BIN);
  //sensor data
  //target data
  file.close();
}


void drawTop() {
  DateTime now = rtc.now();   //pull time
  int offtime = lightonhr + lightonduration;
  //process result
  if (offtime > 23) {
    offtime = offtime - 24;
  }
#ifdef oled
  //Top line has Date, phase , time then two lines
  //date
  display.clearDisplay();
  display.setCursor(0, 0);
  display.print(now.month(), DEC);
  display.print('/');
  display.print(now.day(), DEC);
  display.print('/');
  display.print((now.year() - 2000), DEC);

  //phase icon display
  if (bitRead(SSDstatus, 7) == 1) {
    display.setCursor(60, 0);
    display.print('F');
  }
  else if (bitRead(SSDstatus, 6) == 1) {
    display.setCursor(60, 0);
    display.print('V');
  }
  else if (bitRead(SSDstatus, 5) == 1) {
    display.setCursor(60, 0);
    display.print('G');
  }

  //time
  display.setCursor(80, 0);

  uint8_t hour12 = now.hour() % 12 == 0 ? 12 : now.hour() % 12;
  if (militarytime == 1) {
    display.print(now.hour() , DEC);
  } else display.print(hour12, DEC);
  /*
    if(now.month() == 3 && now.day() >=10){
    display.print((now.hour()+CST), DEC);
    } else if (now.month() >> 3 && now.month() << 11) {
    display.print((now.hour()+CST), DEC);
    } else if (now.month() == 11 && now.day() <<4){
      display.print((now.hour()+CST), DEC);
    }else display.print((now.hour()+DST), DEC);
  */
  display.print(':');
  if (now.minute() < 10) {
    display.print('0');
  }
  display.print(now.minute(), DEC);
  display.print(':');
  if (now.second() < 10) {
    display.print('0');
  }
  display.print(now.second(), DEC);
  // top lines
  display.drawLine(0, 10, display.width() - 1, 10, WHITE);
  display.drawLine(0, 11, display.width() - 1, 11, WHITE);
  /////////////////////////////////////////////////////////
  /////////////////////////////////////////////////////////
}


void drawSelection(void)
{
  if (selection == 0) {
    selection = maxmenu;
  }
  if (selection > maxmenu) {
    selection = 1;
  }
  display.setCursor(00, 13 + (selection * 8));
  display.print("->");
}


void drawHome()
{
  //page 0
  maxmenu = 0;
  selection = 1;
  DateTime now = rtc.now();   //pull time
  int offtime = lightonhr + lightonduration;
  //process result
  if (offtime > 23) {
    offtime = offtime - 24;
  }
  drawTop();
  //this is useful for debug but the led shining is probably indicator enough.
  //Day or Night
  display.setCursor(0, 14);
  display.setTextSize(4);
  if (bitRead(SSDstatus, 4)) {
    display.print("Day");
  }
  else if (!bitRead(SSDstatus, 4)) {
    display.print("ZZz");
  }
  display.setTextSize(1);             // Normal 1:1 pixel scale
  //Right side information
  //debug
  display.print(lightonhr, DEC);
  display.setCursor(72, 22);
  display.print(offtime, DEC);
  display.setCursor(72, 30);
  display.print(SSDstatus, BIN);
  display.setCursor(72, 38);

  if (toggletempF == 1) {
    display.print(((temp1 * 9 / 5) + 32 ), 1);
  } else display.print(temp1, 1);
  display.setCursor(75, 46);
  display.print(humid1, 1);
  display.setCursor(75, 54);
  /*
    display.print("Temp:var$");
    display.setCursor(72, 22);
    display.print("HiTMP:var");
    display.setCursor(72, 30);
    display.print("LoTMP:var");
    display.setCursor(72, 38);
    display.print("RHum:var$");
    display.setCursor(72, 46);
    display.print("HiHum:var");
    display.setCursor(72, 54);
    display.print("LoHum:var");     */
  //display bitfield bitfield for debug
  //split 24 bit in half
  //0-11
  display.setCursor(0, 46);
  //   display.print("Time left:");
  for (int i = 0; i <= 11; i++) {
    int a = bitRead(bitfield, i);
    //Serial.print(a, DEC);
    display.print(a, DEC);
  }
  //  display 12-23 on a new line
  display.setCursor(0, 54);
  for (int i = 12; i <= 23; i++) {
    int a = bitRead(bitfield, i);
    //Serial.print(a, DEC);
    display.print(a, DEC);
  }
  //Time till change
  //    display.print(timelefthr, DEC);
  //    display.print(":");
  //    display.print(timeleftmin, DEC);
  //   display.print(":");
  //   display.print(timeleftsec, DEC);
  //    display.print(seconds_left, DEC);
  display.display();
#endif

#ifdef serial
  Serial.print(now.month(), DEC);
  Serial.print('/');
  Serial.print(now.day(), DEC);
  Serial.print('/');
  Serial.print(now.year(), DEC);
  Serial.print("(");
  Serial.print(daysOfTheWeek[now.dayOfTheWeek()]);
  Serial.print(")");
  Serial.print(now.hour(), DEC);
  Serial.print(':');
  Serial.print(now.minute(), DEC);
  Serial.print(':');
  Serial.print(now.second(), DEC);
  Serial.println();
#endif
}


void drawMenu(void) {
  //page1
  maxmenu = 4;
  drawTop();
  display.setCursor(48, 13);
  display.print("Menu");
  display.setCursor(10, 21);
  display.print("1. Grow Cycle");
  display.setCursor(10, 29);
  display.print("2. Clock");
  display.setCursor(10, 37);
  display.print("3. Datalogging");
  display.setCursor(10, 45);
  display.print("4. Sensors");
  drawSelection();
  display.display();
}


void drawPhase(void) {
  //page2
  maxmenu = 4;
  drawTop();
  display.setCursor(48, 13);
  display.print("Phase");
  display.setCursor(10, 21);
  display.print("1. Start Grow Cycle");
  display.setCursor(10, 29);
  display.print("2. Advance phase.");
  display.setCursor(10, 37);
  display.print("3. Rewind phase.");
  display.setCursor(10, 45);
  display.print("4. Harvest");
  drawSelection();
  display.display();
}


void drawstartCycle(void) {
  //page3
  maxmenu = 2;
  drawTop();
  display.setCursor(0, 13);
  display.print("Skip Germ?");
  display.setCursor(10, 21);
  display.print("Yes");
  display.setCursor(10, 29);
  display.print("No");
  drawSelection();
  display.display();
}


void drawClock(void) {
  //page4
  maxmenu = 4;
  drawTop();
  display.setCursor(0, 13);
  display.print("Clock");
  display.setCursor(10, 21);
  display.print("1. NTP Sync now");
  display.setCursor(10, 29);
  display.print("2. Timezone offset");
  display.setCursor(10, 37);
  if (militarytime == 0) {
    display.print("3. 24 hour");
  }
  if (militarytime == 1) {
    display.print("3. 12 hour");
  }
  drawSelection();
  display.display();
}


void drawDatalog() {
  //page5
  maxmenu = 2;
  drawTop();
  display.setCursor(0, 13);
  display.print("Datalogging");
  display.setCursor(10, 21);
  display.print("1. Log Frequency");
  display.setCursor(10, 29);
  display.print("2.Sensor Selection");
  drawSelection();
  display.display();
}


void drawSensors(void) {
  //page6
  maxmenu = 1;
  drawTop();
  display.setCursor(0, 13);
  display.print("Sensor Config");
  display.setCursor(10, 21);
  if (toggletempF == 0) {
    display.print("1. Display in F");
  } else display.print("1. Display in C");
  display.setCursor(10, 29);
    if (toggletempF == 1) {
    display.print(((temp1 * 9 / 5) + 32 ), 1);
  } else display.print(temp1, 1);
  drawSelection();
  display.display();
}


void drawTimezone(void) {
  //page 7

  maxmenu = 3;
  drawTop();
  display.setCursor(0, 13);

  display.print("Timezone  ");
  display.print(timezone, DEC);
  display.setCursor(10, 21);
  display.print("1. Increase offset");
  display.setCursor(10, 29);
  display.print("2. Decrease offset");
  display.setCursor(10, 37);
  display.print("3. Exit");
  display.setCursor(10, 45);

  drawSelection();
  display.display();
}




void drawScreen(void) {
  switch (page)
  {
    case 0: drawHome();   //p0
      return;
    case 1: drawMenu();   //p1
      return;
    case 2: drawPhase();    //p2
      return;
    case 3: drawstartCycle();   //p3
      return;
    case 4: drawClock();   //p4
      return;
    case 5: drawDatalog();   //p5
      return;
    case 6: drawSensors();    //p6
      return;
    case 7: drawTimezone();
      return;
    default: drawHome();
      page = 0;
      return;
  }
}
void drawPixel(uint16_t x, uint16_t y, uint16_t color);


void makenewstartfile(unsigned int starttime, unsigned int germdur, unsigned int vegdur, unsigned int flowdur )
{
  Serial.print("Initializing SD card...");
  // see if the card is present and can be initialized:


  if (!sd.begin(SD_CHIP_SELECT_PIN)) {
    sd.initErrorHalt();
  }

  if (!file.open("setup.txt", O_RDWR | O_CREAT)) {
    sd.errorHalt(F("open failed"));
  }
  file.println(starttime, DEC);
  file.println(germdur, DEC);
  file.println(vegdur, DEC);
  file.println(flowdur, DEC);
  file.close();
  Serial.println("Wrote to setup.txt");
}
